.Dd September 20, 2010
.Dt mkhomedir 8 CON
.Sh NAME
.Nm mkhomedir
.Nd create user's home directory
.Sh SYNOPSIS
.Nm mkhomedir
.Op Fl -forcehome
.Op Fl -overwrite
.Op Fl -verbose
[
.Op Fl -homedir Ns = Ns  Ar directory
.Op Fl -skel Ns = Ns Ar directory
.Op Fl -uid Ns = Ns Ar UID
.Op Fl -gid Ns = Ns Ar GID
.Bk
.Ar user
.Ek
] ... 
.Sh DESCRIPTION
The
.Nm
command creates user's home directory using a predefined
.Em skeleton
tree normally stored at
.Pa $DEFAULT_SKEL Ns No .
Node's permissions are preserved with the following exceptions:
.Bl -bullet -item
.It
directories are stripped of the SETUID bit,
.It
any other objects have SETUID, SETGID and sticky bits cleared,
.It
if its UID or GID are 0 (root or wheel) they are replaced with user's real
UID or GID, or the ones supplied with the
.Fl -uid
and
.Fl -gid
parameters.
.El
.Pp
This allows automatic creation of files for different services, e.g. make
.Pa public_html
be owned by the
.Li www-data
group. Normally path to the home directory and user/group identifiers 
are read from
the
.Xr passwd 5
database. Any of those values may be overridden with command line options.
Fixing all of the settings
.Fl ( -forcehome , -uid Ns No , and Fl -gid )
causes
.Nm
not to check username against the database at all.

.Pp
This installation of
.Nm
#if ((ALLOW_F==2));then
is not SUID safe. With using the
.Fl -overwrite
option, one can easily replace any file on the system.
#else
has been compiled with SUID safety facilities, that is it will not allow using
the
.Fl -overwrite
option unless caller's effective and real
#if ((ALLOW_F==0));then
UIDs match. If the program has SUID root permissions, only real user root
is then allowed to overwrite any files.
#else
GIDs match. If the program has SGID adm permissions, only user actually
belonging to the group
.Li adm
is allowed to overwrite any files.
#fi
#fi
.Ss OPTIONS
.Bl -tag
.It Fl -homedir Ns = Ns Pa path Ns No , Fl d
Use this path as the base of home directories. It is set to
.Pa $DEFAULT_HOME
by default.
.It Fl -forcehome , H
Normally home directory is obtained either from the
.Xr passwd 5
database if the field is available, or from default path otherwise. This option
enforces the use of home base.
.It Fl -skel Ns No = Ns Pa path Ns No , Fl s
Use this path as the skeleton (template) directory, instead of the default one,
.Pa $DEFAULT_SKEL Ns No .
.It Fl -uid Ns No = Ns Li user Ns No , Fl u
Use this UID instead of the one taken from database.
.It Fl -gid Ns No = Ns Li group Ns No , Fl g
Use this GID instead of the one taken from database.
.It Fl -overwrite , f
Overwrite files and directories. Without this option
.Nm
fails on encountering existing files or directories. Directories are appended
to, all other objects are removed and rewritten.
.It Fl -verbose , v
Print what files are being copied and what permissions with.
.El
.Sh RETURN VALUES
The value of
.Dv EXIT_SUCCESS
(see
.Xr exit 3 )
is returned after processing all users given in the command line. In
case of an error,
.Nm
ends with error code
.Dv EXIT_FAILURE .
.Sh FILES
.Bl -bullet -item
.It
.Pa $DEFAULT_SKEL No - default skeleton directory
.It
.Pa $DEFAULT_HOME No - default home directory
.El 
.Sh EXAMPLES
.Bd -literal -offset 4em
$ ls -Al /etc/skel/
total 18
-rw-r--r-- root root      220 2006-12-11 21:20 .bash_logout
-rw-r--r-- root root     3116 2008-05-12 19:02 .bashrc
-rw-r--r-- root root      675 2008-05-12 19:02 .profile
drwxr-x--- root www-data   48 2010-01-20 13:09 public_html
$ ./mkhomedir -H nobody
$ ls -Al /home/nobody
total 12
-rw-r--r-- nobody nogroup   220 2010-01-20 13:11 .bash_logout
-rw-r--r-- nobody nogroup  3116 2010-01-20 13:11 .bashrc
-rw-r--r-- nobody nogroup   675 2010-01-20 13:11 .profile
drwxr-x--- nobody www-data   48 2010-01-20 13:11 public_html
.Ed
.Sh SEE ALSO
.Xr passwd 5 ,
.Xr group 5 ,
.Xr pam_mkhomedir 8 ,
.Xr useradd 8 ,
.Xr chmod 1 ,
.Xr chown 1
.Sh HISTORY
.Nm
was written overnight by Matlib in September 2010 after having discovered
incapabilities of the already exisiting
.Xr useradd 8
and
.Xr pam_mkhomedir 8
packages.
.Sh BUGS
.Bl -bullet -item
.It
ACLs
.Xr ( acl 5 )
and extended attributes
.Xr ( attr 5 )
are not copied.
.It
Directories cannot be replaced with files in 
.Fl -overwrite
mode.
.Sh AUTHORS
.An Matlib Ad matlib@laser.t28.net
