/*
  strings.c
  Matlib 2010-01-19
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <stdarg.h>

typedef struct
{
  size_t s;
  size_t l;
  char *data;
} String;

#define new_string(__STRING__) \
  ; \
  String __STRING__; \
  string_init (&__STRING__)

void string_init (String *s)
{
  s -> s = 64;
  s -> l = 0;
  s -> data = malloc (s -> s);
  if (s -> data == NULL)
  {
    perror ("malloc");
    exit (EXIT_FAILURE);
  }
}

void string_alloc (String *s, size_t ns)
{
  s -> data = realloc (s -> data, ns);
  if (s -> data == NULL)
  {
    perror ("realloc");
    exit (EXIT_FAILURE);
  }
  s -> s = ns;
}

void string_clr (String *s)
{
  s -> l = 0;
}

void string_str (String *s, char *str)
{
  s -> l = strlen (str);
  if (s -> l + 1 > s -> s)
    string_alloc (s, s -> l + 1);
  strcpy (s -> data, str);
}

void string_astr (String *s, char *str)
{
  size_t l;
  l = strlen (str);
  if (s -> l + l + 1 > s -> s)
    string_alloc (s, s -> l + l + 1);
  strcpy (s -> data + s -> l, str);
  s -> l += l;
}

void string_fmt (String *s, char *format, ...)
{
  va_list ap;
  size_t size;
  va_start (ap, format);
  size = vsnprintf (s -> data, 0, format, ap);
  va_end (ap);
  if (size > s -> s - 1)
    string_alloc (s, size + 1);
  va_start (ap, format);
  vsnprintf (s -> data, s -> s, format, ap);
  va_end (ap);
  s -> l = size;
}

void string_afmt (String *s, char *format, ...)
{
  va_list ap;
  size_t size;
  va_start (ap, format);
  size = vsnprintf (s -> data + s -> l, 0, format, ap);
  va_end (ap);
  if (size > s -> s - s -> l - 1)
    string_alloc (s, size + s -> l + 1);
  va_start (ap, format);
  vsnprintf (s -> data + s -> l, s -> s, format, ap);
  va_end (ap);
  s -> l += size;
}

int string_readlink (String *s, char *name)
{
  int l;
  l = readlink (name, s -> data + s -> l, s -> s - s -> l);
  if (l == -1)
    return errno;
  if (l >= s -> s - s -> l - 1)
  {
    string_alloc (s, s -> l + l + 1);
    l = readlink (name, s -> data + s -> l, s -> s - s -> l);
    if (l == -1)
      return errno;
  }
  s -> l += l;
  s -> data [s -> l] = 0;
  return 0;
}
